# -*- encoding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.colors import LinearSegmentedColormap
from matplotlib.ticker import MultipleLocator, FormatStrFormatter

def PlotRoots(F,zeros):
	""" Function to plot level sets of the vector function F.
	INPUT : 
			F : vector-valued function
			zeros : roots of the function F to identify
	"""

	N=500
	x, y = np.mgrid[-3:3:N*1j, -3:3:N*1j]
	z = F([x,y])
	f1 = z[0, :, :]
	f2 = z[1, :, :]

	fig = plt.figure()
	ax = fig.add_subplot(1, 1, 1)

	cs1 = plt.contour(x, y, f1, [0.0], colors='black', linestyles='dashed')
	cs2 = plt.contour(x, y, f2, [0.0], colors='red')

	plt.plot(0, 0, 'o', color='black')

	lines = [ cs1.collections[0], cs2.collections[0]]
	labels = ['$f(x,y)=0$', '$g(x,y)=0$']

	plt.legend(lines, labels)

	colors = []
	for n, zero in enumerate(zeros):
		p = plt.plot(zero[0], zero[1],'o', markersize=12)
		plt.text(zero[0], zero[1]-0.35, '$\\vec{a}_{%d}$' % (n+1), color=p[0].get_color())
		colors.append(p[0].get_color())
	ax.xaxis.set_major_locator(MultipleLocator(0.5))
	ax.xaxis.set_minor_locator(MultipleLocator(0.1))
	ax.yaxis.set_major_locator(MultipleLocator(0.5))
	ax.yaxis.set_minor_locator(MultipleLocator(0.1))
	ax.xaxis.grid(True,'minor')
	ax.yaxis.grid(True,'minor')
	ax.xaxis.grid(True,'major',linewidth=2)
	ax.yaxis.grid(True,'major',linewidth=2)

	plt.grid()
	plt.savefig('newton_convergence.png')
	plt.show()
	