# -*- encoding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt
from ode45 import ode45
from splitting_parameters import splitting_parameters

# Physikalsichen Konstanten 
L = 2.
m = 3.
g = 1
c = 1

A = -c/(m*L*L)
B = g/L

theta_0 = 2.05
t0 = 0.
T = 1000.

def odefun(t, y):
	r"""
	INPUT:
		t   : 		known time level t=t_n
		y   : 		solution at the known time level t=t_n, i.e. y_n.
					(2d array : y[0] = Winkel, y[1] = Winkelgeschwindigkeit)
	OUTPUT:
		ret :		right hand side of the ODE system.
	"""
	ret = np.empty(2)
	##########################################
	##         TODO: Implementieren         ##
	##         die rechte Seite der ODE     ##
	##########################################
	return ret
def Energien(q,p):
	r""" Total energy of the system.
	INPUT:
		q   : 		First variable of the Hamiltonian function, i.e. Winkel.
		p   :       Second variable of the Hamiltonian function, i.e. Winkelgeschwindigkeit. 
	OUTPUT:
		pot :		Potential energy of the system.
		kin :       Kinetic energy of the system.
		e   : 		Total energy of the system.
	"""
	V = lambda theta: 0.5*c*theta**2 + m*g*L*np.cos(theta)

	pot = V(q)
	kin = 0.5*m*(L**2)*p**2
	e = pot+kin
	return pot, kin, e
def Phi_A(y):
	r""" Operator A:  Phi_A(q,p) = Phi_A(p).
	INPUT:
		y   : 		solution at the known time level t=t_n, i.e. y_n.
					(2d array : y[0] = Winkel, y[1] = Winkelgeschwindigkeit)
	OUTPUT:
		P :			P_a(y).
	"""
	P = np.empty(2)
	##########################################
	##         TODO: Implementieren         ##
	##         den Operator A 		        ##
	##########################################
	return P
def Phi_B(y):
	r""" Operator B: Phi_B(q,p) = Phi_B(q).
	INPUT:
		y   : 		solution at the known time level t=t_n, i.e. y_n.
					(2d array : y[0] = Winkel, y[1] = Winkelgeschwindigkeit)
	OUTPUT:
		P :			P_b(y).
	"""
	P = np.empty(2)
	##########################################
	##         TODO: Implementieren         ##
	##         den Operator B 		        ##
	##########################################
	return P  
def integrate(method, y0, t_end, n_steps):
	r""" Method update.
	INPUT:
	 	method : 		Method to update.
	 	y0 	   : 		Initial value, i.e [winkel, winkelgeschw].
		t_end  : 		End point of the simulation
		n_steps: 		Number of timesteps.
	OUTPUT:
		t 	   :   		Time levels
		y 	   : 		Approximate solutions y (2d array)
	"""
	t, dt = np.linspace(0, t_end, n_steps+1, retstep=True)
	y = np.empty((n_steps+1,) + y0.shape)
	y[0,...] = y0

	for i in range(n_steps):
		y[i+1,...] = method(y[i,...], dt)

	return t, y
def splitting_step(a, b, Phi_a, Phi_b, y0, dt): 
	for a_i, b_i in zip(a, b):
		y0 = y0 + dt*a_i*Phi_a(y0)
		y0 = y0 + dt*b_i*Phi_b(y0)
	return y0
def splitting(splitting_method, Phi_a, Phi_b, y0, t_end, n_steps):
	a, b = splitting_parameters(splitting_method)
	method = lambda y, dt : splitting_step(a, b, Phi_a, Phi_b, y, dt)
	return integrate(method, y0, t_end, n_steps)

def Approximate(T, N, y0, method="BM63") :
	t, y = splitting(method, Phi_A, Phi_B, y0, T, N)
	return t, y
def PlotEnergy(t,tE,tE_ex,labels):
	r""" Totale Energy distortion plot.
	INPUT:
		t 			: m-N dimensional array of N evaluation time points for each of the m methods
		tE 			: m-N dimensional array of total energy computed with m methods
		tE_ex		: Exact Total Energy
		labels 		: labels for the plot
	"""
	m = len(tE)
	fig, ax = plt.subplots(nrows=m, sharex=True)
	for k in range(m):
		ax[k].semilogy(t[k],abs(tE[k]-tE_ex), label=labels[k])
		ax[k].legend(loc="best")

	plt.savefig("abweichungen.pdf")
	plt.show()

if __name__ == "__main__":
	N_SV = 5000
	N_6 = 2500

	y0 = np.array([theta_0, 0])

	# For comparison at sub-task d: uncomment once you have implemented odefun!
	# t_45, y_45 = ode45(odefun,(t0,T), y0)
	# pE_45, kE_45, tE_45 = Energien(y_45[:,0], y_45[:,1])

	#TODO: subtasks b and c!
	# |
	# |
	# v

	# SV 
	t_SV, y_SV = np.linspace(t0,T,N_SV), 2*np.ones((N_SV,2))*y0 #TODO: Compute t_SV and y_SV
	pE_SV, kE_SV, tE_SV = Energien(y_SV[:,0], y_SV[:,1])

	# order 6
	t_6, y_6 = np.linspace(t0,T,N_6), 2*np.ones((N_6,2))*y0 #TODO: Compute t_6 and y_6
	pE_6, kE_6, tE_6 = Energien(y_6[:,0], y_6[:,1])

	# ^
	# |
	# |

	pE_ex, kE_ex, tE_ex = Energien(y0[0],y0[1])

	PlotEnergy([t_45,t_SV,t_6],[tE_45,tE_SV,tE_6],tE_ex,["ode45","SV","O6"])
